package http_server

import (
	"log"
	"net/http"

	"github.com/Vomitblood/cspj-application/server/internal/db"
	"github.com/Vomitblood/cspj-application/server/internal/sql_injection"
)

func healthCheck(w http.ResponseWriter, r *http.Request) {
	w.Header().Set("Content-Type", "application/json")
	w.WriteHeader(http.StatusOK)
	w.Write([]byte(`{"status": "ok"}`))
}

// setup the http server
func ServeApi() {
	http.HandleFunc("/health", healthCheck)
	http.HandleFunc("/health-db", db.DbHealthCheck)
	http.HandleFunc("/setup-demo-db", db.SetupDemoDb)
	http.HandleFunc("/nuke-db", db.NukeDb)
	http.HandleFunc("/fetch-all-users", db.FetchAllUsers)
	http.HandleFunc("/unsecure-register-sql", sql_injection.UnsecureRegisterSql)
	http.HandleFunc("/secure-register-sql", sql_injection.SecureRegisterSql)
	http.HandleFunc("/unsecure-login-sql", sql_injection.UnsecureLoginSql)
	http.HandleFunc("/secure-login-sql", sql_injection.SecureLoginSql)

	log.Println("Server is running on http://localhost:3333")
	if err := http.ListenAndServe(":3333", nil); err != nil {
		log.Fatalf("Failed to start server: %v", err)
	}
}
